/**
 * 
 */

package com.qihoo.appstore.updatelib;

import android.app.Activity;
import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.ResultReceiver;
import android.text.TextUtils;
import android.util.AndroidRuntimeException;
import android.util.Log;

import java.util.concurrent.atomic.AtomicBoolean;

/**
 * @author zhangyong6@360.cn
 */
public class UpdateManager {

    public static final int RESULT_OK = Activity.RESULT_OK;
    public static final int RESULT_FAIL = Activity.RESULT_FIRST_USER + 0x1;

    static final String SDK_VERSION = "2.0";

    private static final String TAG = "UpdateManager";

    private static AtomicBoolean sDebug = new AtomicBoolean(false);

    /**
     * 更新检测回调接口
     */
    public static interface CheckUpdateListener {

        public static final String KEY_ERROR_MSG = "error_msg";
        public static final String KEY_ERROR_CODE = "error_code";
        public static final String KEY_UPDATE_INFO = "UpdateInfo";
        public static final String KEY_EXCEPTION_CLASS_NAME = "ExceptionClassName";
        public static final String KEY_EXCEPTION_STACK_TRACE = "ExceptionStackTrace";

        /**
         * 更新检测函数，当更新检测结果返回时回调
         * 
         * @param status 如果检测成功，则为true，否则为false，注意，检测成功不代表有新版本，有新版本时AppInfo.
         *            isNewVersion也必须为true
         * @param data 附加数据
         *            <p>
         *            附加数据说明：
         *            </p>
         *            <p>
         *            key:KEY_ERROR_MSG value:错误消息，如果有
         *            </p>
         *            <p>
         *            key:KEY_ERROR_CODE value:错误代码，如果值为UpdateManager.RESULT_OK
         *            则表示检测成功。
         *            </p>
         *            <p>
         *            key:KEY_UPDATE_INFO value:AppInfo对象。如果检测成功，则会返回AppInfo对象。
         *            </p>
         *            <p>
         *            key:KEY_EXCEPTION_STACK_TRACE value:如果发生异常，这里包含异常栈信息。
         *            </p>
         */
        void onResult(boolean status, Bundle data);
    }

    /** 是否处于debug模式 */
    public static boolean isDebug() {
        return sDebug.get();
    }

    /** 设置debug模式，debug模式默认为false，设置为true的时候会输出日志，方便调试 */
    public static void setDebug(boolean debug) {
        sDebug.set(debug);
    }

    /**
     * 检测更新
     * 
     * @param context Context
     * @param Activity 检测更新，默认使用弹Dialog提示新版本
     * @throws AndroidRuntimeException 当没有设置渠道号的时候抛出该异常
     */
    public static void checkUpdate(final Context context) {
        checkUpdate(context, false, null);
    }

    /**
     * 检测更新
     * 
     * @param context Context
     * @param listener 检测更新的回调。如果listener不为null,则检测更新不会弹新版本Dialog和Notification。
     * @throws AndroidRuntimeException 当没有设置渠道号的时候抛出该异常
     */
    public static void checkUpdate(final Context context,
            final CheckUpdateListener listener) {
        checkUpdate(context, false, listener);
    }

    /**
     * 检测更新。
     * 
     * @param context Context
     * @param notification 如果为<code>true</code>
     *            ，检测更新完毕则显示通知栏消息而不是对话框，否则显示发现新版本的对话框
     * @throws AndroidRuntimeException 当没有设置渠道号的时候抛出该异常
     */
    public static void checkUpdate(final Context context, final boolean notification) {
        checkUpdate(context, notification, null);
    }

    /**
     * 检测更新。
     * 
     * @param context Context
     * @param notification 如果为<code>true</code>
     *            ，检测更新完毕则显示通知栏消息而不是对话框，否则显示发现新版本的对话框
     * @throws AndroidRuntimeException 当没有设置渠道号的时候抛出该异常
     */
    static void checkUpdate(final Context context, final boolean notification,
            final CheckUpdateListener listener) {
        try {
            String channel = Utils.getChannel(context);
            if (TextUtils.isEmpty(channel)) {
                throw new AndroidRuntimeException(
                        "你必须要在AndroidManifest.xml中添加渠道号<meta-data android:name=\"QIHOO_UPDATE_CHANNEL\"  android:value=\"420010\"/>");
            }
            if (listener != null) {
                CheckUpdateService.checkUpdate(context, context.getPackageName(), channel,
                        notification,
                        new CheckUpdateResultReceiver(new Handler(), listener), false);
            } else {
                CheckUpdateService.checkUpdate(context, context.getPackageName(), channel,
                        notification,
                        null, true);
            }
        } catch (Exception e) {
            if (isDebug()) {
                Log.e(TAG, "checkUpdate", e);
            }
        }
    }

    /**
     * 检测更新。
     * 
     * @param context Context
     * @param notification 如果为<code>true</code>
     *            ，检测更新完毕则显示通知栏消息而不是对话框，否则显示发现新版本的对话框
     * @throws AndroidRuntimeException 当没有设置渠道号的时候抛出该异常
     */
    public static void checkUpdate(final Context context, final boolean notification,
            String packageName,
            String channel,
            final CheckUpdateListener listener) {
        try {
            if (listener != null) {
                CheckUpdateService.checkUpdate(context, packageName, channel,
                        notification,
                        new CheckUpdateResultReceiver(new Handler(), listener), false);
            } else {
                CheckUpdateService.checkUpdate(context, packageName, channel,
                        notification,
                        null, true);
            }
        } catch (Exception e) {
            if (isDebug()) {
                Log.e(TAG, "checkUpdate", e);
            }
        }
    }

    private static class CheckUpdateResultReceiver extends ResultReceiver {
        private CheckUpdateListener mListener;

        public CheckUpdateResultReceiver(Handler handler, CheckUpdateListener listener) {
            super(handler);
            mListener = listener;

        }

        @Override
        protected void onReceiveResult(int resultCode, Bundle resultData) {
            super.onReceiveResult(resultCode, resultData);
            if (mListener != null) {
                if (resultData == null) {
                    resultData = new Bundle();
                }
                resultData.putInt(CheckUpdateListener.KEY_ERROR_CODE, resultCode);
                mListener.onResult(resultCode == UpdateManager.RESULT_OK, resultData);
            }
        }
    }

    /**
     * 调用手机助手更新，改函数将会调起助手的软件详情页面。
     * 
     * @param context
     * @param packageName 你的应用的包名。
     * @param autoDownload 是否自动下载，如果为true，则在打开详情页后，助手会自动开始下载并安装。
     * @return 返回值
     *         <p>
     *         -1:参数不正确，-2：手机助手未安装，-3：未知异常，0：调用成功
     *         </p>
     */
    public static int startAppStoreToUpdate(Context context, String packageName,
            boolean autoDownload) {
        if (context == null) {
            return -1;
        }

        if (TextUtils.isEmpty(packageName)) {
            return -1;
        }
        return Utils.startAppStoreToUpdate(context, packageName, autoDownload);
    }

    /**
     * 执行直接更新
     * 
     * @param context
     * @param updateInfo 更新对象
     */
    public static boolean doUpdate(Context context, AppInfo updateInfo) {
        try {
            if (updateInfo.isNewVersion) {
                Context context2 = context.getApplicationContext();
                String file = Utils.getUpdateFile(context2, updateInfo.downloadUrl,
                        updateInfo.packageName);
                PackageManager packageManager = context2.getPackageManager();
                ApplicationInfo applicationInfo = packageManager.getApplicationInfo(
                        updateInfo.packageName,
                        0);
                String title = (String) packageManager.getApplicationLabel(applicationInfo);
                CheckUpdateService.startDownload(context2, updateInfo.downloadUrl, file, title, 1,
                        null,
                        null);
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            if (UpdateManager.isDebug()) {
                Log.w(TAG, "doDirectDownload fail ", e);
            }
            return false;
        }
    }

    /**
     * 执行省流量更新或者高速更新。
     * 
     * @param context
     * @param updateInfo 更新对象
     */
    public static boolean doPatchUpdate(Context context, AppInfo updateInfo) {
        try {
            if (updateInfo.isNewVersion) {
                Context context2 = context.getApplicationContext();
                if (Utils.isPackageInstalled(context2, Utils.APPSTORE_PKGNAME)) {
                    Utils.startAppStoreToUpdate(context2, updateInfo.packageName, true);
                } else {
                    String url = Utils.getAppStoreDownloadUrl(context2);
                    String title = RHelper.getString(context2, "appstore_title");
                    String file = Utils.getUpdateFile(context2, url, Utils.APPSTORE_PKGNAME);
                    CheckUpdateService.startDownload(context2, url, file, title,
                            3,
                            null,
                            updateInfo.packageName);
                }
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            if (UpdateManager.isDebug()) {
                Log.w(TAG, "doPatchDownload fail ", e);
            }
            return false;
        }
    }
}
